#ifndef __UART_H__
#define __UART_H__

#include <stdint.h>
#include <stdbool.h>
#include "nrf.h"

/** Some definitions needed for the ported console functions */
#define UART_MODE_GENERIC_POLLING 0

/* Newline character sequences */
#define UART_NEWLINE_CRLF          "\r\n"
#define UART_NEWLINE_CR            '\r'
#define UART_NEWLINE_LF            '\n'
#define UART_NEWLINE_CRLF_NUMERIC  "\x0D\x0A"  /* Hardcoded ASCII values for CRLF */
#define UART_NEWLINE_CR_NUMERIC    "\x0D"      /* It is possible, though unlikely, that */
#define UART_NEWLINE_LF_NUMERIC    "\x0A"      /* \r and \n do not equal these values. */
/* Default newline style */
#define UART_NEWLINE_DEFAULT       UART_NEWLINE_CRLF_NUMERIC

/* Newline style for input */
#ifndef UART_NEWLINE_INPUT
  /* UART_NEWLINE_INPUT was not defined in console_config.h */
  #define UART_NEWLINE_INPUT      UART_NEWLINE_CR
#endif
  
/* Newline style for output */
#ifndef UART_NEWLINE_OUTPUT
  /* UART_NEWLINE_OUTPUT was not defined in console_config.h */
  #define UART_NEWLINE_OUTPUT     UART_NEWLINE_DEFAULT
#endif


/** Available Baud rates for UART. */
typedef enum
{
    UART_BAUD_2K4 = 0,        ///< 2400 baud
    UART_BAUD_4K8,            ///< 4800 baud
    UART_BAUD_9K6,            ///< 9600 baud
    UART_BAUD_14K4,           ///< 14.4 kbaud
    UART_BAUD_19K2,           ///< 19.2 kbaud
    UART_BAUD_28K8,           ///< 28.8 kbaud
    UART_BAUD_38K4,           ///< 38.4 kbaud
    UART_BAUD_57K6,           ///< 57.6 kbaud
    UART_BAUD_76K8,           ///< 76.8 kbaud
    UART_BAUD_115K2,          ///< 115.2 kbaud
    UART_BAUD_230K4,          ///< 230.4 kbaud
    UART_BAUD_250K0,          ///< 250.0 kbaud
    UART_BAUD_500K0,          ///< 500.0 kbaud
    UART_BAUD_1M0,            ///< 1 mbaud
    UART_BAUD_TABLE_MAX_SIZE  ///< Used to specify the size of the baudrate table.
} uart_baudrate_t;

/** @brief The baudrate devisors array, calculated for standard baudrates.
    Number of elements defined by ::HAL_UART_BAUD_TABLE_MAX_SIZE*/
#define UART_BAUDRATE_DEVISORS_ARRAY    { \
    0x0009D000, 0x0013B000, 0x00275000, 0x003BA000, 0x004EA000, 0x0075F000, 0x009D4000, \
    0x00EBE000, 0x013A9000, 0x01D7D000, 0x03AFB000, 0x03FFF000, 0x075F6000, 0x10000000  }

/** @brief This function initializes the UART.
 *
 * @param baud_rate - the baud rate to be used, ::uart_baudrate_t.
 *
 */
void uart_init(uart_baudrate_t const baud_rate);

/** @brief Find number of characters in the UART receive buffer
 *
 * This function returns the number of characters available for reading
 * in the UART receive buffer.
 * 
 * @return Number of characters available
 */
bool uart_chars_available(void);

/** Function to write a character to the UART transmit buffer.
 * @param ch Character to write
 */
void uart_putchar(uint8_t ch);

/** Function to read a character from the UART receive buffer.
 * @return Character read
 */
uint8_t uart_getchar(void);

/** Function to write and array of byts to UART.
 *
 * @param buf - buffer to be printed on the UART.
 * @param len - Number of bytes in buffer.
 */
void uart_write_buf(uint8_t const *buf, uint32_t len);

/**@brief Logging function, used for formated output on the UART.
 */
void uart_logf(const char *fmt, ...);

#ifdef USE_UART_LOG_DEBUG
    #define LOG_DEBUG(F, ...) (uart_logf("LOG : %s: %d: " F "\r\n", __FILE__, __LINE__, ##__VA_ARGS__))
#else
    #define LOG_DEBUG(F, ...) (void)__NOP()
#endif
#ifdef USE_UART_LOG_INFO
    #define _LOG_INFO(F, ...) (uart_logf(F, ##__VA_ARGS__))
    #define LOG_INFO(F, ...) (uart_logf(F "\r\n", ##__VA_ARGS__))
#else
    #define LOG_INFO(F, ...) (void)__NOP()
#endif

uint8_t uart_getline(uint8_t * string, uint8_t max_len);
void simple_uart_put(uint8_t cr);
void simple_uart_config(uint8_t rts_pin_number, uint8_t txd_pin_number, uint8_t cts_pin_number, uint8_t rxd_pin_number, bool hwfc);
#endif // __UART_H__
